SystemOrganization addCategory: #'Pier-Magma'!

PRPersistency subclass: #PRMagmaPersistency
	instanceVariableNames: 'snapshot contexts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaPersistency commentStamp: 'kph 9/14/2006 03:33' prior: 0!
I am stored in the PRMagmaRepository as the base object for each PRKernel.
I store the current #kernel
I store a historical #snapshot
I store a log of changes, a collection of #contexts, whose commands can be replayed to obtain a complete history.

----
In Pier, PRContext objects may cache #properties in relation to the current seaside display context session and components. Persisting these runtime non-model objects is not a good idea since they may include references to the current magma session which will break the persistency. To avoid this we send #magmaCopy to all instances of PRContext, before saving them into the log of #contexts. This also applies to the 'context's command's answer context'.

If one of these persisted contexts is used and by chance has its properties repopulated, then PRContext-#maPreSerialize zaps #properties as well before the object would be serialised and persisted again. 



!

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/28/2006 20:40'!
doing: aCommand critical: aBlock

"this is the essential 'write' interface from pier to magma."

self kernel critical: [ self session commit: aBlock ].! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:18'!
do: aBlock 
	 contexts do: [ :each | aBlock value: (each setKernel: self kernel) ]! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 9/8/2006 14:04'!
exportToMagma: m

"already a magma persistence strucure"

^self! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 8/15/2006 05:20'!
importFromPRPersistency: persistency

"this is our opportunity to munge datastructures into magma-useful things, such as MagmaCollections for historical records etc."

^self! !

!PRMagmaPersistency methodsFor: 'cache' stamp: 'kph 9/29/2006 00:15'!
internalLink: aLink

"we are informed of an internal link so we may cache it"
 ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/14/2006 02:11'!
log: aContext

	 contexts add: aContext magmaCopy ! !

!PRMagmaPersistency methodsFor: 'cache' stamp: 'kph 9/29/2006 00:14'!
pathLookup: aString startingAt: aStructure fallback: aBlock  

^nil! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:24'!
realizeKernel

"I return the kernel instance from the repository under my kernel's name (my own kernel is likely to be a stub)"
 
| persisted |

persisted := self session root 
			storeAt: (self kernel name) 
			ifAbsent: [ self error: ('not found in database: ', self kernel name) ].

^persisted kernel! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:19'!
reverseDo: aBlock 
	contexts reverseDo: [ :each | aBlock value: (each setKernel: self kernel) ]! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/28/2006 04:57'!
session

^WACurrentSession value! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:30'!
setKernel: aKernel
	kernel := aKernel.
	self snapshot.! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:23'!
snapshot
	snapshot := self kernel veryDeepCopy.
	contexts := OrderedCollection new! !

!PRPersistency methodsFor: '*pier-magma' stamp: 'kph 8/3/2006 22:03'!
exportToMagma: magmaPersistency

^magmaPersistency importFromPRPersistency: self! !

!PRPersistency methodsFor: '*pier-magma' stamp: 'kph 9/29/2006 01:12'!
internalLink: aLink

"we are informed of an internal link so we may cache it"
 ! !

!PRPersistency methodsFor: '*pier-magma' stamp: 'kph 9/29/2006 01:12'!
pathLookup: aString startingAt: aStructure fallback: aBlock  

^nil! !

!PRCommand methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:28'!
postMagmaCopy

	answer ifNotNil: [ answer := answer magmaCopy ].! !

WAComponent subclass: #PRMagmaControlPanel
	instanceVariableNames: 'magmaSession text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaControlPanel class methodsFor: 'repository init' stamp: 'kph 9/9/2006 00:47'!
copy: kernelName to: repositoryPath

"not tested!!"

"assuming that a kernel is available in memory, or online in another repository.
 deep copy this kernel into the new repository.

e.g.
PRMagmaRepository create: 'backup'.
PRMagmaRepository copy: 'pier' to: 'backup'.
"

|  kernel newKernel session |

kernel := PRKernel instanceNamed: kernelName.

session := self classMagmaSession openLocal:repositoryPath.
session connectAs: 'copy'.
 
kernel critical: [ newKernel := kernel veryDeepCopy ].

session commit: [ session root store at: newKernel name put: newKernel persistency].

session disconnect ;
		closeRepository

"
 
"! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 01:56'!
magmaRootClass

^ PRMagmaRepository ! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 00:48'!
magmaRootDescription

^ 'The Pier Content Management System uses a PRMagmaRepository as a root object. This maintains a Dictionary of PRMagmaPersistency''s, and a Dictionary of properties'! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:31'!
on: magmaSession

^self new
	magmaSession: magmaSession;
	yourself.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:13'!
deleteImageKernel: k

self doubleConfirm: ('delete kernel: ', k name)
	thenDo: [ PRKernel instances remove: k ] ! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:15'!
deleteRepositoryKernel: name

self doubleConfirm: ('delete kernel from the repository: ',  name)
	thenDo: [ 
				magmaSession commit: [ magmaSession root store removeKey: name ]
		].! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:13'!
doubleConfirm: message thenDo: aBlock

((self confirm: 'Are you sure you want to ', message) and: 
	[ self confirm: 'Are you REALLY sure you want to ', message ])
	ifTrue: [ 
				^aBlock value.
	].
! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:08'!
loadKernel: name 

PRKernel instances detect: [ :k | k name = name ] ifNone: [ ^magmaSession root makeLocalStub: name ].

self inform: 'Kernel named: ', name, ' already exists.'.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:59'!
magmaSession: ms

magmaSession := ms! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/19/2006 18:03'!
persist: kernel

kernel critical: [ magmaSession commit: [ kernel root. magmaSession root persist: kernel ] ]! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/23/2006 15:49'!
renderContentOn: html

html heading level: 3; with: 'Pier-Magma Control Panel'.

magmaSession ifNil: [ ^self renderNoMagmaSessionOn: html ].

html table: [
	html tableRow: [  
	     html tableData: [
			self renderOn: html table: 'image' with: PRKernel instances using: [:item | self renderKernelInstance: item on: html ].
			
			html textInput size: 10;
				 on: #text of: self. 
			html submitButton callback: [ PRKernel named: self text ] ; text: 'new kernel'.
		].
		html tableData: [
			self renderOn: html table: 'repository' with: self repository store using: [:item | self renderRepository: item on: html ].
		].
	]
]
! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/21/2006 04:53'!
rendererClass

^ WARenderCanvas.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:14'!
renderKernelInstance: k on: html

html tableData: [ html text: k name ].

html tableData: [ 
	
	html text: '(',k persistency name, ')'.

].
html tableData: [ 
		html small: [ html anchor callback: [ self deleteImageKernel: k] ; text: '(delete)' ] 
].
html tableData: [ 

(k persistency class ~= PRMagmaPersistency)
 	ifTrue: [
		html submitButton callback: [ self persist: k ] ; text: 'Persist' 
	]

]! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/19/2006 13:24'!
renderNoMagmaSessionOn: html 

html bold: 'No MagmaSession currently available'.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/21/2006 04:54'!
renderOn: html table: title with: items using: aBlock

html table: [

	html tableRow: [
		html tableData: [
		html tableHeading: [ html text: title ].
].
	].

	items do: [ :item | 

		html tableRow: [
			 aBlock value: item  
		].

	].

]! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 05:09'!
renderRepository: aPersistency on: html

| name |

name := aPersistency kernel name.

html tableData: [ html text: name   ].

html tableData: [ 
	PRKernel instances detect: [ :k | k name = name ] 
	ifNone: [  
		html submitButton callback: [self loadKernel: name ]; text: 'load'.
	].
	html small: [ html anchor callback: [self deleteRepositoryKernel: name ]; text: '(delete)' ].	
]

! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 03:00'!
repository

^magmaSession root! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:20'!
text

^text! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:21'!
text:   aString

text := aString! !

!WAMagmaSession methodsFor: '*pier-magma' stamp: 'kph 9/15/2006 14:53'!
magmaAbortOnRequest.

magmaSession ifNotNil: [ magmaSession abort. ].! !

!WAMagmaSession methodsFor: '*pier-magma' stamp: 'kph 9/15/2006 14:56'!
responseForRequest: aRequest
	
self magmaAbortOnRequest.

^super responseForRequest: aRequest! !

!PRStructure methodsFor: '*pier-magma' stamp: 'kph 9/25/2006 04:20'!
maPreserialize

"ensure that there are no empty PRChildren persisted since
reading empties back in generates needless database realisations."

| childrenDecoration | 

childrenDecoration := self decorationOfClass: self childrenDecorationClass ifAbsent: [ ^self ].

childrenDecoration isEmpty ifTrue: [
	self removeDecoration: childrenDecoration.
].
! !

!PRStructure methodsFor: '*pier-magma' stamp: 'kph 9/25/2006 04:15'!
maWantsPreSerialization 
	^ true! !

!MaReadStrategy methodsFor: '*pier-magma' stamp: 'bf 3/27/2006 19:15'!
forVariableNamed: aString onAny: aClass readToDepth: anInteger

	(aClass allInstVarNames includes: aString)
		ifFalse:
			[ MagmaUserError signal: aString , ' is not an attribute of ' , aClass name ].
	(depthSpecifications
		at: aClass
		ifAbsentPut: [ self depthDictionaryOfSize: aClass instSize ])
			at: aString
			put: anInteger! !

!MaReadStrategy methodsFor: '*pier-magma' stamp: 'cmm 1/26/2005 22:21'!
instVarMapFor: aMaClassDefinition
	"Answer an Array with an instVarMap of aMaClassDefinition according to user-declared read depths."

	| specifiedDepths answer |
	(aMaClassDefinition isDefined
	and: [ depthSpecifications includesKey: aMaClassDefinition classObject ])
		ifTrue:
			[ specifiedDepths _ depthSpecifications at: aMaClassDefinition classObject.
			specifiedDepths isInteger
				ifTrue: [ answer _ specifiedDepths ]
				ifFalse:
					[ answer _ self depthArrayOfSize: aMaClassDefinition namedInstSize.
					specifiedDepths keysAndValuesDo:
						[ : eachInstVarName : eachDepth |
						answer
							at: (aMaClassDefinition allInstVarNames indexOf: eachInstVarName)
							put: eachDepth ] ] ].
	^ answer! !

!MaReadStrategy methodsFor: '*pier-magma' stamp: 'cmm 6/28/2004 14:56'!
onAny: aClass readToDepth: anInteger

	depthSpecifications
		at: aClass
		put: anInteger! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:26'!
maPreserialize

"ensure that properties are always zapped before persisting"
	properties := nil.! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:25'!
maWantsPreSerialization 
	^ true! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:27'!
postMagmaCopy

	properties := nil.	 
	self setCommand: self command magmaCopy.! !

!PRPathLookup class methodsFor: '*pier-magma' stamp: 'kph 9/28/2006 20:46'!
noCacheStart: aStructure path: aString

 ^ self new stream: aString readStream;
							lookup: aStructure.
	 ! !

!PRPathLookup class methodsFor: '*pier-magma' stamp: 'kph 9/28/2006 20:47'!
start: aStructure path: aString

^(aStructure withPersistenceDo: [ :cache | 
	cache 	pathLookup: aString 
			startingAt: aStructure 
			fallback: [ self noCacheStart: aStructure path: aString ] ]) 
			
	ifNil: [ self noCacheStart: aStructure path: aString ].
	 ! !

PRObject subclass: #PRMagmaRepository
	instanceVariableNames: 'store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaRepository commentStamp: '<historical>' prior: 0!
I am used as the root object in a magma repository. 
I am subclassed from PRObject so that I may have properties.

----
To Create Repository:

There is a GUI provided in the Magma-seaside package which is available within seaside/config to create a repository.

To manually create a repository directory called 'magma-pier' in the default directory:

MagmaRepositoryController create: 'magma-pier' root: PRMagmaRepository new.

Again there is a GUI available in the 'MagmaControlPanel' for persisting PRKernels.
Manually we tell the default repository to persist an existing pier kernel referenced by its name like so.  

| session |
session := MagmaSession openLocal: 'magma-pier'.
session connectAs: 'local-squeak'.
session commit: [ 
PRMagmaRepository persist: (PRKernel instanceNamed: 'home').
].
session disconnectAndClose.

The existing persistence structure is migrated into the magma repository. The framework for acheiving this is provided in the protocol 'translating'. Although at present this does nothing, it is here in order to allow PRMagmaPersistency to tweak the model that it it persisting. For example we may choose to use indexed MagmaCollections to store the history. The Pier framework need not be aware of any Magma specifics, hence the double dispatch of exportToMagma methods which refer back to Magma specific importFrom methods on PRMagmaPersistency.

 

!

!PRMagmaRepository class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:51'!
isAbstract

^false! !

!PRMagmaRepository methodsFor: 'configuration' stamp: 'kph 9/28/2006 03:35'!
buildReadStrategyOn: strategy

"set the read strategy for this application"

strategy minimumDepth: 0;
	"	forVariableNamed: 'store' onAny: PRMagmaRepository readToDepth: 2 ;"
		forVariableNamed: 'kernel' onAny: PRImagePersistency readToDepth: 1;
		onAny: PRKernel readToDepth: 1 ;
		forVariableNamed: 'decorations' onAny: PRDecorated readToDepth: 1; 
		forVariableNamed: 'title' onAny: PRStructure readToDepth: 1; 

		forVariableNamed: 'children' onAny: PRChildren readToDepth: 
			(self readPathSkeletonInOneHit ifTrue: [ 1 ] ifFalse: [ 0 ]);

		onAny: PRDocumentItem readToDepth: 1; "reads all page items in one hit"

		onAny: PRComponent readToDepth: 1;
yourself.! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 05:24'!
magmaControlPanelOn: magmaSession

^PRMagmaControlPanel on: magmaSession! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/29/2006 05:30'!
persist: kernel  

"we make a new kernel instance to go into the repository,
we assign its persistency manager, and root structure.

This process includes an opportunity to munge its model if needed
or to import from another persistency scheme.

Then we turn the image PRKernel into a stub which is used to obtain the persisted kernel via #realize.
"

| persistedKernel | 

	persistedKernel := PRKernel new name: kernel name; yourself.

	persistedKernel persistency: ( kernel persistency exportToMagma: PRMagmaPersistency new ).
	persistedKernel root: kernel root.

	store at: kernel name put:  kernel persistency.

	"stub out image kernel"
	kernel persistency: PRMagmaPersistency new.
	kernel root: nil.! !

!PRMagmaRepository methodsFor: 'configuration' stamp: 'kph 9/23/2006 16:03'!
readPathSkeletonInOneHit

"the path skeleton is all of the data needed to traverse the heirarchical path of the pier kernel
 - if this is set to false then each level down in the path traversal will be a separate object realisation
 - this may be desireable for small to medium sizred wiki's"

^false! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/8/2006 14:02'!
store

^store ifNil: [ store := Dictionary new ]! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/28/2006 06:16'!
storeAt: name

^self store at: name! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/28/2006 06:16'!
storeAt: name ifAbsent: aBlock

^self store at: name ifAbsent: aBlock! !

!PRObject methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:47'!
magmaCopy

^ self shallowCopy postMagmaCopy! !

!PRObject methodsFor: '*pier-magma' stamp: 'kph 9/29/2006 01:00'!
withPersistenceDo: aBlock

| context |

(context := PRCurrentContext value) ifNil: [ ^nil ].

^ aBlock value: context kernel persistency.! !

!PRKernel methodsFor: '*pier-magma' stamp: 'kph 9/24/2006 03:01'!
maPreserialize

"ensure that mutex is zapped before persisting, then at least it will be in a predictable state when relaised."
	mutex := nil.! !

!PRKernel methodsFor: '*pier-magma' stamp: 'kph 9/24/2006 03:02'!
maWantsPreSerialization 
	^ true! !
